/***************************************************************************
 *            interface.c
 *
 *  Fri Oct 21 22:34:23 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>

#include "interface-fun.h"
#include "lists-fun.h"
#include "menage_db-fun.h"

/*	Pliki naglowkowe funkcji zwrotnyvh	*/
#include "main_win-callbacks.h"
#include "about_dlg-callbacks.h"
#include "property_dlg-callbacks.h"
#include "book_type_dlg-callbacks.h"
#include "m_book_win-callbacks.h"
#include "menage_db_win-callbacks.h"
#include "s_book_win-callbacks.h"
#include "m_volume_win-callbacks.h"


#define GUI_PATH PACKAGE_DATA_DIR"/glibrary/glade/glibrary.glade"
/*#define	GUI_PATH "/home/tomek/GLibrary/Rozwojowa/glibrary.glade"*/

/*#define ICON_PATH "/home/tomek/GLibrary/Rozwojowa/pixmaps/glibrary.png"*/
#define ICON_PATH PACKAGE_DATA_DIR"/glibrary/pixmaps/glibrary.png"

/*
 *	Funkcja laduje plik XML z konfiguracja GUI sprawdza czy:
 *	- plik XML wygenerowany przez program Glade znajduje sie w katalogu
 *	  zdefiniowanym podczas instalacji
 *	- sprawdza obsluge 'gmodule' wymaganej do auto-podlaczenia sygnalow zwrotnych
 *	- sprawdza czy pliki graficzne programu znajduja sie na swoim miejscu
 *	  jezeli ich tam nie ma program zostanie uruchomiony i pokaze sie ostrzezenie
 *	- ostatecznie zaladowane zostanie okno 'main_win'
 */

gboolean	load_gui_xml			(void)	{
	GError		*error;
	
	error = NULL;
	
	icon = gdk_pixbuf_new_from_file (ICON_PATH, &error);
	
	if (!g_file_test (GUI_PATH, G_FILE_TEST_EXISTS))	{
		GtkWidget	*window;
		gint		status;
		
		window = gtk_message_dialog_new_with_markup	(NULL,
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
									_("<b><span size=\"large\">Critical error</span></b>"));
		
		gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (window),
									_("GLibrary can't load Glade XML file which contain GUI\n"
									"configuration. Please check if this file exist:"
									"<i>%s</i> ."),
									GUI_PATH);
		
		status = gtk_dialog_run (GTK_DIALOG (window));
		switch (status)	{
			case GTK_RESPONSE_CLOSE:
				gtk_widget_destroy (window);
				break;
			case GTK_RESPONSE_DELETE_EVENT:
				gtk_widget_destroy (window);
				break;
			default:
				gtk_widget_destroy (window);
		}
		
		return FALSE;
	}
	else if (g_module_supported () == FALSE)	{
		GtkWidget	*window;
		gint		status;
		
		window = gtk_message_dialog_new_with_markup	(NULL,
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
									_("<span size=\"large\"><b>Critical error</b></span>"));
		
		gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (window),
									_("GModule (GLib component) isn't supported by your operating\n"
									"system, for more informations see <i>http://www.gtk.org/</i>."));
		
		status = gtk_dialog_run (GTK_DIALOG (window));
		switch (status)	{
			case GTK_RESPONSE_CLOSE:
				gtk_widget_destroy (window);
				break;
			case GTK_RESPONSE_DELETE_EVENT:
				gtk_widget_destroy (window);
				break;
			default:
				gtk_widget_destroy (window);
		}
		
		return FALSE;
	}

	else if (icon == NULL)	{
		printf ("Failed to load pixbuf file: %s: %s\n",
               ICON_PATH, error->message);
		g_error_free (error);
		return TRUE;
	}
	
	return TRUE;
}

/*
 *	Funkcja niszczy okno, ktorego elementem jest argument 'widget'.
 *	Zwalniana jest tez pamiec zajmowana przez zmiena typu GladeXML, ktora
 *	przechowuje iformacje na temat niszczonego okna
 */

void	destroy_window		(GtkWidget	*widget)	{
	GtkWidget	*window;
	GladeXML	*xml;
	
	window = gtk_widget_get_toplevel (widget);
	xml = glade_get_widget_tree (window);
	gtk_widget_destroy (window);
	g_object_unref (xml);
}

/*
 *	Funkcja tworzy okno 'main_win'
 */


GtkWidget	*create_main_win (void)	{
	GladeXML			*xml;
	GtkListStore 		*store;
    GtkCellRenderer 	*renderer;
    GtkTreeViewColumn 	*column;
	GtkWidget 			*widget;
	GtkWidget			*window;
	
	xml = glade_xml_new (GUI_PATH, "main_win", NULL);
	window = glade_xml_get_widget (xml, "main_win");
	main_win = window;
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	glade_xml_signal_autoconnect (xml);
	
	/*	Tworzymy liste 'main_list'	*/
	create_main_list ();

	/*	Tworzymy liste 'books_list'	*/
	create_main_win_m_books_list ();
	
	//create_books_list (0);
	/*	Ustawiamy wskaznik, tak zeby okno glowne bylo dostepne dla calego
	 *	programu	*/
	
	return	window;
}

/*
 *	Funkcja buduje okno 'about_dlg' na podstawie pliku XML programu Glade
 */

GtkWidget	*create_property_dlg		(void)	{
	GtkWidget	*window;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "property_dlg", NULL);
	window = glade_xml_get_widget (xml, "property_dlg");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	glade_xml_signal_autoconnect (xml);
	
	create_property_dlg_books_list (window);
	
	return window;
}

/*
 *	Funkcja buduje okno 'about_dlg' na podstawie pliku XML programu Glade
 */

GtkWidget	*create_about_dlg		(void)	{
	GtkWidget	*window;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "about_dlg", NULL);
	window = glade_xml_get_widget (xml, "about_dlg");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	glade_xml_signal_autoconnect (xml);
	
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (window), VERSION);
	gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (window), icon);
	
	return window;
}

/*
 *	Funkcja tworzy okno 'search_win'
 */

GtkWidget	*create_search_win	(void)	{
	GtkWidget	*window;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "search_win", NULL);
	glade_xml_signal_autoconnect (xml);
	window = glade_xml_get_widget (xml, "search_win");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	
	create_search_books_list (window);
	
	return window;
}

/*
 *	Funkcja tworzy okno 's_book_win'
 *	Argument 'mode':
 *	- 0 - ustawia tytul okna na "Dodawanie ksiazki"
 *	- 1 - ustawia tytul okna na "Edycja ksiazki"
 */

GtkWidget	*create_s_book_win	(gint	mode)	{
	GtkWidget	*window;
	GtkWidget	*widget;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "s_book_win", NULL);
	glade_xml_signal_autoconnect (xml);
	window = glade_xml_get_widget (xml, "s_book_win");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	
	switch (mode)	{
		case 0:
			gtk_window_set_title (GTK_WINDOW (window), _("Add book"));
			widget = glade_xml_get_widget (xml, "s_book_update_but");
			gtk_widget_hide (widget);
			break;
		case 1:
			gtk_window_set_title (GTK_WINDOW (window), _("Edit book"));
			widget = glade_xml_get_widget (xml, "s_book_save_but");
			gtk_widget_hide (widget);
			break;
	}
	
	return window;
}

/*
 *	Funkcja tworzy okno 'm_book_win'
 *	Argument 'mode':
 *	- 0 - ustawia tytul okna na "Dodawanie ksiazki"
 *	- 1 - ustawia tytul okna na "Edycja ksiazki"
 */

GtkWidget	*create_m_book_win	(gint	mode)	{
	GtkWidget	*window;
	GtkWidget	*widget;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "m_book_win", NULL);
	glade_xml_signal_autoconnect (xml);
	window = glade_xml_get_widget (xml, "m_book_win");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	
	switch (mode)	{
		case 0:
			gtk_window_set_title (GTK_WINDOW (window), _("Add book"));
			widget = glade_xml_get_widget (xml, "m_book_update_but");
			gtk_widget_hide (widget);
			break;
		case 1:
			gtk_window_set_title (GTK_WINDOW (window), _("Edit book"));
			widget = glade_xml_get_widget (xml, "m_book_save_but");
			gtk_widget_hide (widget);
			break;
	}
	
	/*	Tworzymy liste 'books_list'	*/
	
	create_m_volume_list (window);
	
	return window;
}

/*
 *	Funkcja tworzy okno 'menage_db_win'
 */

GtkWidget	*create_menage_db_win	(void)	{
	GtkWidget	*window;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "menage_db_win", NULL);
	glade_xml_signal_autoconnect (xml);
	window = glade_xml_get_widget (xml, "menage_db_win");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	
	/*	Tworzymy liste	*/
	create_menage_db_list (window);
	
	/*	Musimy ustawic jakis typ edytowanej tabeli bazy danych, pierwsza opcja
	 *	w ramce 'Menu' sa 'Autorzy', wiec przyjmiemy ich za domyslna wartosc	*/
	set_menage_db_win_data_type (window, 0);
	
	return window;
}

/*
 *	Funkcja tworzy okno 'm_volume_dlg_dlg'
 *	Laczymy reczne sygnaly poniewaz musimy miec dostep z okna 'm_volume_win'
 *	do okna nadrzednego 'm_book_win', a konkretnie do listy w oknie 'm_book_win'
 *	Argument 'widget' musi byc dowolnym widgetem z okna 'm_book_win'.
 */

GtkWidget	*create_m_volume_dlg	(GtkWidget	*parent,
									 gint		mode)	{
	GtkWidget	*window;
	GtkWidget	*widget;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "m_volume_dlg", NULL);
	window = glade_xml_get_widget (xml, "m_volume_dlg");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
										 
	glade_xml_signal_connect_data (xml, 
								   "on_m_volume_add_but_clicked",
								   G_CALLBACK (on_m_volume_add_but_clicked),
								   parent);	
	glade_xml_signal_connect 	(xml, 
							  	"on_m_volume_first_translator_en_changed",
								 G_CALLBACK (on_m_volume_first_translator_en_changed));	


	glade_xml_signal_connect_data 	(xml, 
							  		"on_m_volume_cancel_but_clicked",
									 G_CALLBACK (on_m_volume_cancel_but_clicked),
									 parent);									 

	glade_xml_signal_connect_data 	(xml, 
							 	 	"on_m_volume_win_delete_event",
									 G_CALLBACK (on_m_volume_win_delete_event),
									 parent);	
									 
	glade_xml_signal_connect_data 	(xml, 
							 	 	"on_m_volume_update_but_clicked",
									 G_CALLBACK (on_m_volume_update_but_clicked),
									 parent);							 
	
	switch (mode)	{
		case 0:
			gtk_window_set_title (GTK_WINDOW (window), _("Add volume"));
			widget = glade_xml_get_widget (xml, "m_volume_update_but");
			gtk_widget_hide (widget);
			break;
		case 1:
			gtk_window_set_title (GTK_WINDOW (window), _("Edit volume"));
			widget = glade_xml_get_widget (xml, "m_volume_add_but");
			gtk_widget_hide (widget);
			break;
	}
	
	return window;
}

/*
 *	Funkcja tworzy okno 'book_type_dlg'
 */

GtkWidget	*create_book_type_dlg	(void)	{
	GtkWidget	*window;
	GladeXML	*xml;
	
	xml = glade_xml_new (GUI_PATH, "book_type_dlg", NULL);
	glade_xml_signal_autoconnect (xml);
	window = glade_xml_get_widget (xml, "book_type_dlg");
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	
	return window;
}

/*
 *	Funkcja tworzy okno z ostrzezeniem, informacja, lub bledem.
 *	Argument 'first_text' oznacza komunikat, a argument 'sec_text' oznacza
 *	tresc komunikatu.
 *	Wartosci zmiennej 'dlg_mode':
 *		0 - informacja
 *		1 - ostrzezeniem
 *		2 - blad
 */

void		show_message_dlg		(gchar		*first_text,
									 gchar		*sec_text,
									 gint		dlg_mode)
{
	GtkWidget	*window;
	gchar		string	[64];
	
	g_sprintf (string, "<span size=\"large\"><b>%s</b></span>", first_text);
	
	switch (dlg_mode)	{
		case 0:
			window = gtk_message_dialog_new (NULL,
											 GTK_DIALOG_DESTROY_WITH_PARENT,
        	    	                         GTK_MESSAGE_INFO,
            	    	                     GTK_BUTTONS_OK,
											 NULL);
			gtk_window_set_icon (GTK_WINDOW (window), icon);
			gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (window), string);
			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (window), "%s", sec_text);
			gtk_dialog_run (GTK_DIALOG (window));
			gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);
			gtk_widget_destroy (window);
			break;
		case 1:
			window = gtk_message_dialog_new (NULL,
											 GTK_DIALOG_DESTROY_WITH_PARENT,
        	    	                         GTK_MESSAGE_WARNING,
            	    	                     GTK_BUTTONS_CLOSE,
											 NULL);
			gtk_window_set_icon (GTK_WINDOW (window), icon);
			gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (window), string);
			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (window), "%s", sec_text);
			gtk_dialog_run (GTK_DIALOG (window));
			gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);
			gtk_widget_destroy (window);
			break;
		case 2:
			window = gtk_message_dialog_new (NULL,
											 GTK_DIALOG_DESTROY_WITH_PARENT,
        	    	                         GTK_MESSAGE_ERROR,
            	    	                     GTK_BUTTONS_CLOSE,
											 NULL);
			gtk_window_set_icon (GTK_WINDOW (window), icon);
			gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (window), string);
			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (window), "%s", sec_text);
			gtk_dialog_run (GTK_DIALOG (window));
			gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);
			gtk_widget_destroy (window);
			break;
		default:
			g_printf ("(ERROR) Bad value of \'dlg_mode\' passed to \'show_message_dlg ()\'\n");
		}
}

/*
 *	Funkcja twprzy standartowe okno okno pytania, dostarczane przez GTK+
 *	Argument 'first_text' oznacza pytanie, a argument 'sec_test' oznacza 
 *	dodatkowy opis.
 *	Funkcja zwraca wartosc TRUE jezeli zostanie wcisniety przycisk 'Tak' i 
 *	wartosc FALSE gdy zostanie wcisniety przycisk 'Nie' lub gdy okno zostanie
 *	zniszczone.
 *	Wartosci zmiennej 'icon_type':
 *		0 - pytanie
 *		1 - informacja
 *		2 - ostrzezeniem
 *		3 - blad
 */

gboolean	show_question_dlg			(gchar		*first_text,
										 gchar		*sec_text,
										 gint		icon_type)
{
	GtkWidget	*window;
	gint		response_id;
	gboolean	status;	
	gchar		string	[64];
	
	g_sprintf (string, "<span size=\"large\"><b>%s</b></span>", first_text);

	switch (icon_type)	{
		case 0:
			window = gtk_message_dialog_new (NULL,
									 GTK_DIALOG_DESTROY_WITH_PARENT,
        	    	                 GTK_MESSAGE_QUESTION,
            	    	             GTK_BUTTONS_YES_NO,
									 NULL);
			break;
		case 1:
			window = gtk_message_dialog_new (NULL,
									 GTK_DIALOG_DESTROY_WITH_PARENT,
        	    	                 GTK_MESSAGE_INFO,
            	    	             GTK_BUTTONS_YES_NO,
									 NULL);
			break;
		case 2:
			window = gtk_message_dialog_new (NULL,
									 GTK_DIALOG_DESTROY_WITH_PARENT,
        	    	                 GTK_MESSAGE_WARNING,
            	    	             GTK_BUTTONS_YES_NO,
									 NULL);
			break;
		case 3:
			window = gtk_message_dialog_new (NULL,
									 GTK_DIALOG_DESTROY_WITH_PARENT,
        	    	                 GTK_MESSAGE_ERROR,
            	    	             GTK_BUTTONS_YES_NO,
									 NULL);
			break;
	}
	gtk_window_set_icon (GTK_WINDOW (window), icon);
	gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (window), string);
	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (window), "%s", sec_text);
	response_id = gtk_dialog_run (GTK_DIALOG (window));
	
	switch (response_id)	{
		case GTK_RESPONSE_DELETE_EVENT:
			gtk_widget_destroy (window);
			status = FALSE;
			break;
		case GTK_RESPONSE_YES:
			gtk_widget_destroy (window);
			status = TRUE;
			break;
		case GTK_RESPONSE_NO:
			gtk_widget_destroy (window);
			status = FALSE;
			break;
		default:
			status = FALSE;
	}
	
	return status;
}

/*
 * Funkcja ustawia wartosc paska stanu okna 'main_win'
 */

void		set_statusbar_text	(gchar	*text)	{
	GtkWidget	*widget;
	GladeXML	*xml;
	gint		context_id;

	xml = glade_get_widget_tree (main_win);
	widget = glade_xml_get_widget (xml, "main_statbar");
	
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (widget), text);
	gtk_statusbar_push (GTK_STATUSBAR (widget), context_id, text);
}

/*
 *	Funkcja aktywuje/dezaktywuje widget. 
 *	Wartosci 'status':
 *		TRUE  - widget aktywny
 *		FALSE - widget nie aktywny
 */

void	set_widget_sensinitive (GtkWidget	*widget,
								gchar		*widget_name,
								gboolean	status)	
{
	GtkWidget	*tmp;
	GladeXML	*xml;
	
	xml = glade_get_widget_tree (widget);
	tmp = glade_xml_get_widget (xml, widget_name);
	
	gtk_widget_set_sensitive (tmp, status);
}
