/***************************************************************************
 *            main.c
 *
 *  Mon Aug 22 10:11:06 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <glib/gi18n-lib.h>

#include "interface-fun.h"
#include "main_win-callbacks.h"
#include "database-fun.h"
#include "book-fun.h"

int
main (int argc, char *argv[])
{
	GtkWidget	*main_win;
	gboolean	status;

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif
	
	gtk_set_locale ();
	gtk_init (&argc, &argv);
	
	/*
	 *	Zaczynamy zabawe z odpalaniem GBiblioteki, sprawdzamy czy baza daych 
	 *	istnieje, sprawdzamy jej wersje oraz sprawdzamy jak wyglada sprawa z 
	 *	obsluga interfejsu
	 */
	
	status = load_gui_xml ();
	
	if (status)	{
		GtkWidget	*widget;
		
		widget = create_main_win ();
		gtk_widget_show (widget);
		
		status = check_db_exist ();
		if (status)	{
			open_db ();
			status = check_db_version ();
			if (status)	{
				load_books_titles ();
				set_statusbar_text (_("GLibrary is ready to work."));
			}	
			else	{
				show_message_dlg (_("Unsupported database version"),
					_("This version of GLibrary can't work with this database, "
					  "please try to use other versionof GLibrary."), 2);
			}			
		}	
		else	{
			create_db ();
			open_db ();	
			load_books_titles ();			
			set_statusbar_text (_("GLibrary is ready to work."));
		}			
		
		gtk_main ();
	}
	return 0;
}
